/*
 * (c) Copyright Robin Hill 2004
 *
 * For license terms, see the file COPYING in this directory.
 *
 * Wrapper for wxFFile using customised Tell(), Seek() and Length()
 * functions changed for 64-bit support, and with added Resize() function.
 *
 */

#ifdef __WINDOWS__
	typedef INT64 fr_off_t;
#else
	typedef off_t fr_off_t;
#endif

#include <wx/ffile.h>

class frFFile
{
public:
	frFFile()
		{ wxfile = new wxFFile(); }
	frFFile(const wxChar *filename, const wxChar *mode = wxT("r"))
		{ wxfile = new wxFFile(filename, mode); }
	frFFile(FILE *fp)
		{ wxfile = new wxFFile(fp); }

	bool Open(const wxChar *filename, const wxChar *mode = wxT("r"))
		{ return wxfile->Open(filename, mode); }
	bool Close()
		{ return wxfile->Close(); }

	void Attach(FILE *fp, const wxString& name = wxT(""))
		{ wxfile->Attach(fp, name); }
	void Detach()
		{ wxfile->Detach(); }
	FILE *fp() const
		{ return wxfile->fp(); }

	bool ReadAll(wxString *str)
		{ return wxfile->ReadAll(str); }
	size_t Read(void *pBuf, size_t nCount)
		{ return wxfile->Read(pBuf, nCount); }
	size_t Write(const void *pBuf, size_t nCount)
		{ return wxfile->Write(pBuf, nCount); }
	bool Write(const wxString& s, wxMBConv& conv = wxConvLibc)
		{ return wxfile->Write(s, conv); }
	bool Flush()
		{ return wxfile->Flush(); }

	bool Seek(fr_off_t ofs, wxSeekMode mode = wxFromStart);
	bool SeekEnd(fr_off_t ofs = 0) { return Seek(ofs, wxFromEnd); }
	fr_off_t Tell() const;
	fr_off_t Length() const;
	bool Resize(const fr_off_t size);

	bool IsOpened() const
		{ return wxfile->IsOpened(); }
	bool Eof() const
		{ return wxfile->Eof(); }
	bool Error() const
		{ return wxfile->Error(); }
	const wxString& GetName() const
		{ return wxfile->GetName(); }

	~frFFile() { Close(); delete wxfile; }

private:
	// copy ctor and assignment operator are private because it doesn't
	// make sense to copy files this way: attempt to do it will provoke
	// a compile-time error.
	frFFile(const frFFile&);
	frFFile& operator=(const frFFile&);

	wxFFile	*wxfile;
};
