/*
 * (c) Copyright Robin Hill 2002
 *
 * For license terms, see the file COPYING in this directory.
 */

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

// For all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers
#ifndef WX_PRECOMP
	#include <wx/wx.h>
#endif

#if !wxUSE_THREADS
    #error "This program requires thread support!"
#endif


#ifdef __WINDOWS__
    #include <windows.h>
#endif /* __WINDOWS__ */

#if wxUSE_DRAG_AND_DROP
	#include <wx/dnd.h>
#endif

#include <wx/notebook.h>
#include <wx/socket.h>
#include <wx/wxhtml.h>
#include <wx/fs_zip.h>
#include "wx_extra_functions.h"
#include "defines.h"
#include "frFFile.h"
#include "fr_backend.h"
#include "fr_gui.h"
#include "fr_threads_gui.h"

IMPLEMENT_APP(FileRepairApp)

#if defined(__WXGTK__) || defined(__WXMOTIF__)
	#include "filerepair.xpm"
#endif

// The event table.  Most of the events are pretty obvious.  Event calls
// from the backend process are handled as menu events.
BEGIN_EVENT_TABLE(FileRepairDlg, wxFrame)
	EVT_BUTTON(FR_Quit, FileRepairDlg::OnQuit)
	EVT_BUTTON(FR_FileBrowse, FileRepairDlg::OnFileBrowse)
	EVT_BUTTON(FR_ChecksumBrowse, FileRepairDlg::OnChecksumBrowse)
	EVT_BUTTON(FR_FileHash, FileRepairDlg::OnFileHashClick)
	EVT_BUTTON(FR_ChecksumCreate, FileRepairDlg::OnChecksumCreateClick)
	EVT_BUTTON(FR_ChecksumVerify, FileRepairDlg::OnChecksumVerifyClick)
	EVT_BUTTON(FR_ClientStart, FileRepairDlg::OnClientStartClick)
	EVT_BUTTON(FR_ServerStart, FileRepairDlg::OnServerStartClick)
	EVT_BUTTON(FR_ClientStop, FileRepairDlg::OnStopClick)
	EVT_BUTTON(FR_ServerStop, FileRepairDlg::OnStopClick)
	EVT_TEXT(FR_FileName, FileRepairDlg::edtFileNameChange)
	EVT_TEXT(FR_ChecksumFile, FileRepairDlg::edtChecksumFileChange)
	EVT_TEXT(FR_ServerAddress, FileRepairDlg::CheckButtons)
	EVT_NOTEBOOK_PAGE_CHANGING(FR_ModeTab, FileRepairDlg::tabModeChanging)
	EVT_MENU(FR_MenuFileExit, FileRepairDlg::OnQuit)
	EVT_MENU(FR_MenuHelpContents, FileRepairDlg::HelpContents)
	EVT_MENU(FR_MenuHelpIndex, FileRepairDlg::HelpIndex)
	EVT_MENU(FR_MenuHelpAbout, FileRepairDlg::About)
	EVT_MENU(THREAD_STARTED, FileRepairDlg::ChangeState)
	EVT_MENU(THREAD_ERROR, FileRepairDlg::ErrorMessage)
	EVT_MENU(THREAD_ALERT, FileRepairDlg::AlertMessage)
	EVT_MENU(THREAD_STATUS, FileRepairDlg::StatusMessage)
	EVT_MENU(THREAD_PROGRESS, FileRepairDlg::ProgressMessage)
	EVT_MENU(THREAD_HASH, FileRepairDlg::HashMessage)
	EVT_MENU(THREAD_SRVINFO, FileRepairDlg::ServerMessage)
	// We have to implement this to force closing
	// the dialog when the 'x' button is pressed
	EVT_CLOSE(FileRepairDlg::OnClose)
END_EVENT_TABLE()

bool FileRepairApp::OnInit()
{
// Debug settings
#ifdef __WXDEBUG__
// Output debug messages to the frgui_errors.txt file
	FILE *errors;
	errors = fopen("frgui_errors.txt", "w");
	wxLog *logger = new wxLogStderr(errors);
	wxLog::SetActiveTarget(logger);
// Add another trace mask for function entry/exit
//	wxLog::AddTraceMask("frTRACE_functions");
//	wxLog::AddTraceMask("frTRACE_net");
//	wxLog::AddTraceMask(wxTRACE_MemAlloc);
	wxLog::AddTraceMask("frTRACE_debug");
#endif

// Create the GUI dialog box
	FileRepairDlg *dialog = new FileRepairDlg(wxT("FileRepair"),
		wxPoint(-1, -1), wxSize(489,533));

// Show it and tell the application that it's our main window
	dialog->Show(TRUE);
	SetTopWindow(dialog);

	return TRUE;
};

// If we have drag and drop then set up a handler for dropped files.  This
// just takes the filename and puts it into the box.
#if wxUSE_DRAG_AND_DROP
bool FileNameDrop::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
{
	if (filenames.GetCount() != 1) return FALSE;
	m_pOwner->SetValue(filenames[0]);
	return TRUE;
};
#endif

// Dialog constructor
FileRepairDlg::FileRepairDlg(const wxString& title, const wxPoint& pos, const wxSize& size)
	: wxFrame((wxFrame *)NULL, -1, title, pos, size, (wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION))
{
	int             x;
	wxMenu          *mnuFile,
	                *mnuHelp;
	wxMenuBar       *menubar = new wxMenuBar();
	bool            helpmenu = FALSE;
	wxString        FileHashes[4] = {wxT("eDonkey"), wxT("MD5"), wxT("CRC32"), wxT("TTH")};

#if wxUSE_STREAMS && wxUSE_ZIPSTREAM && wxUSE_ZLIB 
// Help files
	wxFileSystem::AddHandler(new wxZipFSHandler);
	frHelp = new wxHtmlHelpController();
	frHelp->UseConfig(wxConfig::Get());
	wxString app_path = wxFindAppPath(wxTheApp->argv[0], wxGetCwd(), wxT("APPDIR"));
	if (! app_path.IsEmpty()) {
		if (frHelp->AddBook(app_path + wxT("/filerepair.htb")))
			helpmenu = TRUE;
	}
#endif

// The menu bar.  Currently there's just File/Exit and Help/About.
// This is mainly because I needed somewhere to put an about box!
// If the help files were found then Help/Contents and Help/Index are added.
	mnuFile = new wxMenu();
	mnuFile->Append(FR_MenuFileExit, wxT("E&xit"));
	menubar->Append(mnuFile, wxT("&File"));
	mnuHelp = new wxMenu();
	if (helpmenu) {
		mnuHelp->Append(FR_MenuHelpContents, wxT("&Contents"));
		mnuHelp->Append(FR_MenuHelpIndex, wxT("&Index"));
		mnuHelp->AppendSeparator();
	}
	mnuHelp->Append(FR_MenuHelpAbout, wxT("&About FileRepair"));
	menubar->Append(mnuHelp, wxT("&Help"));


// Set/Clear the parameters
	ChkFile.Clear();
	InputFile.Clear();
	ServerAddress.Clear();
	passive = FALSE;
	blocksize = DEF_BLOCKSIZE / 1024;
	port = DEF_PORTNUM;
	SecondThread = NULL;

// Set the dialog icon
	SetIcon(wxICON(filerepair));

// Create and position controls in the dialog
// Start with the permanent features
	wxPanel *bgPanel = new wxPanel(this, FR_MainPanel, wxPoint(0,0), wxSize(489, 513));
	wxStaticText *labFileName = new wxStaticText(bgPanel, FR_FileNameLab, wxT("File:"), wxPoint(16, 16));
	x = labFileName->GetSize().GetWidth();
	edtFileName = new wxTextCtrl(bgPanel, FR_FileName, wxEmptyString , wxPoint(20 + x, 12), wxSize(381 - x, 21));
#if wxUSE_DRAG_AND_DROP
	edtFileName->SetDropTarget(new FileNameDrop(edtFileName));
#endif
	btnFileBrowse = new wxButton(bgPanel, FR_FileBrowse, wxT("Browse"), wxPoint(408, 10), wxSize(57, 25));
	tabMode = new wxNotebook(bgPanel, FR_ModeTab, wxPoint(8, 40), wxSize(465, 305));
	edtStatus = new wxStaticText(bgPanel, FR_Status, wxEmptyString, wxPoint(8, 352));
	barProgress = new wxGauge(bgPanel, FR_ProgressBar, 100, wxPoint(8, 384), wxSize(465, 41), wxGA_HORIZONTAL | wxGA_SMOOTH | wxGA_PROGRESSBAR);
	btnQuit = new wxButton(bgPanel, FR_Quit, wxT("Quit"), wxPoint(180, 440), wxSize(121, 41));

// Now the standalone tab
	tabStandalone = new wxPanel(tabMode, FR_StandaloneTab);
	tabMode->AddPage(tabStandalone, wxT("Standalone"), TRUE);
	boxCreateHash = new wxStaticBox(tabStandalone, FR_CreateHash, wxT("Create Hash:"), wxPoint(8, 8), wxSize(441, 105));
	labFileHash = new wxStaticText(tabStandalone, FR_FileHashLab, wxT("Type:"), wxPoint(80, 41));
	chcFileHash = new wxChoice(tabStandalone, FR_FileHashType, wxPoint(110, 37), wxSize(81, 23), 4, FileHashes);
	btnFileHash = new wxButton(tabStandalone, FR_FileHash, wxT("Create Hash"), wxPoint(248, 32), wxSize(101, 33));
	edtHashResult = new wxTextCtrl(tabStandalone, FR_HashResult, wxEmptyString, wxPoint(16, 80), wxSize(425, 21), wxTE_READONLY);
	edtHashResult->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
	boxCRC32 = new wxStaticBox(tabStandalone, FR_CRC32, wxT("CRC-32:"), wxPoint(8, 124), wxSize(441, 137));
	labChecksumFile = new wxStaticText(tabStandalone, FR_ChecksumFileLab, wxT("Checksum File:"), wxPoint(24, 143));
// Since the text sizes are different on different platforms I fiddle the width
// and positioning of some elements slightly
	x = labChecksumFile->GetSize().GetWidth();
	edtChecksumFile = new wxTextCtrl(tabStandalone, FR_ChecksumFile, wxEmptyString, wxPoint(28 + x, 139), wxSize(353 - x, 21));
#if wxUSE_DRAG_AND_DROP
	boxCRC32->SetDropTarget(new FileNameDrop(edtChecksumFile));
#endif
	btnChecksumBrowse = new wxButton(tabStandalone, FR_ChecksumBrowse, wxT("Browse"), wxPoint(384, 137), wxSize(57, 25));
	btnChecksumCreate = new wxButton(tabStandalone, FR_ChecksumCreate, wxT("Create"), wxPoint(120, 172), wxSize(81, 33));
	btnChecksumVerify = new wxButton(tabStandalone, FR_ChecksumVerify, wxT("Verify"), wxPoint(248, 172), wxSize(81, 33));
	labBlocksize = new wxStaticText(tabStandalone, FR_BlocksizeLab, wxT("Blocksize:"), wxPoint(180, 220));
	x = labBlocksize->GetSize().GetWidth();
	edtBlocksize = new wxTextCtrl(tabStandalone, FR_Blocksize, wxString::Format(wxT("%d"), DEF_BLOCKSIZE / 1024), wxPoint(184 + x, 216), wxSize(89 - x, 21), 0, wxTextValidator(wxFILTER_NUMERIC));
	labBlocksizeMul = new wxStaticText(tabStandalone, FR_BlocksizeMul, wxT("k"), wxPoint(278, 220));

// The Client tab
	tabClient = new wxPanel(tabMode, FR_ClientTab);
	tabMode->AddPage(tabClient, wxT("Client"), FALSE);
	boxClientSetup = new wxStaticBox(tabClient, FR_ClientSetup, wxT("Setup:"), wxPoint(8, 8), wxSize(441, 121));
	labServerAddress = new wxStaticText(tabClient, FR_ServerAddressLab, wxT("Server Address:"), wxPoint(24, 28));
	x = labServerAddress->GetSize().GetWidth();
	edtServerAddress = new wxTextCtrl(tabClient, FR_ServerAddress, wxEmptyString, wxPoint(28 + x, 24), wxSize(413 - x, 21));
	labClientPort = new wxStaticText(tabClient, FR_ClientPortLab, wxT("Port:"), wxPoint(128, 76));
	x = labClientPort->GetSize().GetWidth();
	edtClientPort = new wxTextCtrl(tabClient, FR_ClientPort, wxString::Format(wxT("%d"), DEF_PORTNUM), wxPoint(132 + x, 72), wxSize(79 - x, 21), 0, wxTextValidator(wxFILTER_NUMERIC));
	labClientBlocksize = new wxStaticText(tabClient, FR_ClientBlocksizeLab, wxT("Blocksize:"), wxPoint(242, 76));
	x = labClientBlocksize->GetSize().GetWidth();
	edtClientBlocksize = new wxTextCtrl(tabClient, FR_ClientBlocksize, wxString::Format(wxT("%d"), DEF_BLOCKSIZE / 1024), wxPoint(246 + x, 72), wxSize(93 - x, 21), 0, wxTextValidator(wxFILTER_NUMERIC));
	labClientBlocksizeMul = new wxStaticText(tabClient, FR_ClientBlocksizeMul, wxT("k"), wxPoint(340, 76));
	chkClientPassive = new wxCheckBox(tabClient, FR_ClientPassive, wxT("Client is passive mode server"), wxPoint(128, 104), wxSize(225, 17));
	btnClientStart = new wxButton(tabClient, FR_ClientStart, wxT("START"), wxPoint(88, 144), wxSize(121, 41));
	btnClientStop = new wxButton(tabClient, FR_ClientStop, wxT("STOP"), wxPoint(248, 144), wxSize(121, 41));
	btnClientStop->Enable(FALSE);
	labClientResults = new wxStaticText(tabClient, FR_ClientResults, wxEmptyString, wxPoint(8, 200));

// And finally the Server tab
	tabServer = new wxPanel(tabMode, FR_ServerTab);
	tabMode->AddPage(tabServer, wxT("Server"), FALSE);
	boxServerSetup = new wxStaticBox(tabServer, FR_ServerSetup, wxT("Setup:"), wxPoint(8, 8), wxSize(441, 121));
	labServerPort = new wxStaticText(tabServer, FR_ServerPortLab, wxT("Port:"), wxPoint(188, 52));
	x = labServerPort->GetSize().GetWidth();
	edtServerPort = new wxTextCtrl(tabServer, FR_ServerPort, wxString::Format(wxT("%d"), DEF_PORTNUM), wxPoint(192 + x, 48), wxSize(76 - x, 21), 0, wxTextValidator(wxFILTER_NUMERIC));
	chkServerPassive = new wxCheckBox(tabServer, FR_ServerPassive, wxT("Client is passive mode server"), wxPoint(128, 104), wxSize(225, 17));
	btnServerStart = new wxButton(tabServer, FR_ServerStart, wxT("START"), wxPoint(88, 144), wxSize(121, 41));
	btnServerStop = new wxButton(tabServer, FR_ServerStop, wxT("STOP"), wxPoint(248, 144), wxSize(121, 41));
	btnServerStop->Enable(FALSE);
	labServerResults = new wxStaticText(tabServer, FR_ServerResults, wxEmptyString, wxPoint(8, 200));

// Set the FileHash choice to the first entry
    chcFileHash->SetSelection(0);
// Enable/Disable the various buttons & boxes
	CheckButtons((wxCommandEvent &) wxEVT_NULL);
// Show the menu bar
	SetMenuBar(menubar);
}

// Stop the user from changing tabs while a process is running
void FileRepairDlg::tabModeChanging(wxNotebookEvent& event)
{
	if ((SecondThread) && (SecondThread->IsRunning())) event.Veto();
}

// Check whether we need to update button status after the user
// has changed the Checksum File box.  Also change the global
// variable for the checksum file
void FileRepairDlg::edtChecksumFileChange(wxCommandEvent& WXUNUSED(event))
{
	bool    current,
	        old;

	old = ! ChkFile.IsEmpty();

	if (edtChecksumFile->GetLineLength(0) == 0) {
		ChkFile.Clear();
		current = FALSE;
	} else {
		ChkFile = edtChecksumFile->GetValue();
		current = TRUE;
	}

	if (current != old) CheckButtons((wxCommandEvent &) wxEVT_NULL);
}

// Check whether we need to update button status after the user
// has changed the Filename box.  Also change the global variable
// for the filename
void FileRepairDlg::edtFileNameChange(wxCommandEvent& WXUNUSED(event))
{
	bool    current,
	        old;

	old = ! InputFile.IsEmpty();

	if (edtFileName->GetLineLength(0) == 0) {
		InputFile.Clear();
		current = FALSE;
	} else {
		InputFile = edtFileName->GetValue();
		current = TRUE;
	}

	if (current != old) CheckButtons((wxCommandEvent &) wxEVT_NULL);
}

// Enable or disable buttons depending on whether the necessary information
// has been provided by the user
void FileRepairDlg::CheckButtons(wxCommandEvent& WXUNUSED(event))
{
	btnFileHash->Enable(! InputFile.IsEmpty());
	chcFileHash->Enable(! InputFile.IsEmpty());
	btnChecksumCreate->Enable(! (InputFile.IsEmpty() || ChkFile.IsEmpty()));
	btnChecksumVerify->Enable(! (InputFile.IsEmpty() || ChkFile.IsEmpty()));
	btnServerStart->Enable(! InputFile.IsEmpty());
	btnClientStart->Enable(! (InputFile.IsEmpty() || (edtServerAddress->GetLineLength(0) == 0)));
}

// This adds an icon to the dialog
void FileRepairDlg::SetIcon(const wxIcon& icon)
{
	m_icon = icon;
#ifdef __WIN95__
	if ( m_icon.Ok() )
		SendMessage((HWND) GetHWND(), WM_SETICON,
			(WPARAM)TRUE, (LPARAM)(HICON) m_icon.GetHICON());
#endif
}

// Show the standard file open dialog and require that the file already
// exists.  Put the selection into the Filename box
void FileRepairDlg::OnFileBrowse(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog *filDlg = new wxFileDialog(this, wxT("Choose a file"), wxEmptyString, wxEmptyString, wxT("Any file (*.*)|*.*"), (wxOPEN | wxFILE_MUST_EXIST));
	if (filDlg->ShowModal() == wxID_OK) {
		edtFileName->SetValue(filDlg->GetPath());
	}
	delete filDlg;
}

// Show the standard file open dialog but don't require that the file already
// exists.  Put the selection into the Checksum file box
void FileRepairDlg::OnChecksumBrowse(wxCommandEvent& WXUNUSED(event))
{
	wxFileDialog *filDlg = new wxFileDialog(this, wxT("Choose a checksum file"), wxEmptyString, wxEmptyString, wxT("Checksum File(*.crc)|*.crc|Any file (*.*)|*.*"), wxOPEN);
	if (filDlg->ShowModal() == wxID_OK) {
		edtChecksumFile->SetValue(filDlg->GetPath());
	}
	delete filDlg;
}

void FileRepairDlg::OnFileHashClick(wxCommandEvent& WXUNUSED(event))
{
	edtHashResult->SetValue(wxEmptyString);
	switch (chcFileHash->GetSelection()) {
        case 0:
        	StartThread(MODE_FH_ED2K);
        	break;
        case 1:
        	StartThread(MODE_FH_MD5);
        	break;
        case 2:
        	StartThread(MODE_FH_CRC);
        	break;
        case 3:
        	StartThread(MODE_FH_TTH);
        	break;
        default:
            break;
    }
}

void FileRepairDlg::OnChecksumCreateClick(wxCommandEvent& WXUNUSED(event))
{
// Make sure we have a blocksize >= 1, otherwise give an error
	if (edtBlocksize->GetValue().ToLong(&blocksize)) {
		if (blocksize < 1) {
			wxMessageDialog	*errdlg = new wxMessageDialog(this, wxT("Blocksize is invalid"), wxT("Error"), wxOK | wxCENTRE | wxICON_HAND);
			errdlg->ShowModal();
			delete errdlg;
		} else
			StartThread(MODE_CRC_CREATE);
	}
}

void FileRepairDlg::OnChecksumVerifyClick(wxCommandEvent& WXUNUSED(event))
{
	StartThread(MODE_CRC_VERIFY);
}

void FileRepairDlg::OnClientStartClick(wxCommandEvent& WXUNUSED(event))
{
	labClientResults->SetLabel(wxEmptyString);
	ServerAddress = edtServerAddress->GetValue();
	passive = chkClientPassive->GetValue();
// Check the blocksize >=1 and that the port is valid
	if (edtClientPort->GetValue().ToLong(&port) && edtClientBlocksize->GetValue().ToLong(&blocksize)) {
		if (blocksize < 1) {
			wxMessageDialog	*errdlg = new wxMessageDialog(this, wxT("Blocksize is invalid"), wxT("Error"), wxOK | wxCENTRE | wxICON_HAND);
			errdlg->ShowModal();
			delete errdlg;
		} else if ((port < 1) || (port > 65535)) {
			wxMessageDialog	*errdlg = new wxMessageDialog(this, wxT("Port is invalid"), wxT("Error"), wxOK | wxCENTRE | wxICON_HAND);
			errdlg->ShowModal();
			delete errdlg;
		} else
			StartThread(MODE_CLIENT_START);
	}
}

void FileRepairDlg::OnServerStartClick(wxCommandEvent& WXUNUSED(event))
{
	labServerResults->SetLabel(wxEmptyString);
	passive = chkServerPassive->GetValue();
// Check the port is a valid port
	if (edtServerPort->GetValue().ToLong(&port)) {
		if ((port < 1) || (port > 65535)) {
			wxMessageDialog	*errdlg = new wxMessageDialog(this, wxT("Port is invalid"), wxT("Error"), wxOK | wxCENTRE | wxICON_HAND);
			errdlg->ShowModal();
			delete errdlg;
		} else
			StartThread(MODE_SERVER_START);
	}
}

void FileRepairDlg::OnStopClick(wxCommandEvent& WXUNUSED(event))
{
	SecondThread->Delete();
	SecondThread = NULL;
}

// This actually starts the backend processing thread
void FileRepairDlg::StartThread(wxUint32 mode)
{
	SecondThread = new ProcessThread(this, mode);
	if (SecondThread->Create() != wxTHREAD_NO_ERROR) {
		wxLogError(wxT("Can't create thread!"));
		SecondThread->Delete();
		SecondThread = NULL;
	} else {
		SecondThread->Run();
	}
}

// This is called from the backend thread and disables most of
// the functions in the user interface whilst the backend processing
// is being done.  It also changes the cursor to indicate this.
// Once the backend process is complete this function is called again to
// reverse the changes.
void FileRepairDlg::ChangeState(wxCommandEvent& event)
{
	bool state;

	if (event.GetString().Cmp(wxT("Started")) == 0) {
		state = FALSE;
		wxBeginBusyCursor();
		wxCursor test = * wxSTANDARD_CURSOR;
		btnQuit->SetCursor(test);
		btnServerStop->SetCursor(test);
		btnClientStop->SetCursor(test);
	} else if (event.GetString().Cmp(wxT("Stopped")) == 0) {
		state = true;
		SecondThread = NULL;
		wxEndBusyCursor();
	} else {
		return;
	}

	edtFileName->Enable(state);
	btnFileBrowse->Enable(state);

	switch (event.GetInt()) {
		case MODE_FH_ED2K:
		case MODE_FH_MD5:
		case MODE_FH_CRC:
		case MODE_FH_TTH:
			if (! state) edtHashResult->Clear();
		case MODE_CRC_CREATE:
		case MODE_CRC_VERIFY:
			btnFileHash->Enable(state);
			chcFileHash->Enable(state);
			edtChecksumFile->Enable(state);
			btnChecksumBrowse->Enable(state);
			if (! ChkFile.IsEmpty()) {
				btnChecksumCreate->Enable(state);
				btnChecksumVerify->Enable(state);
			}
			break;
		case MODE_CLIENT_START:
			edtServerAddress->Enable(state);
			edtClientPort->Enable(state);
			edtClientBlocksize->Enable(state);
			chkClientPassive->Enable(state);
			btnClientStart->Enable(state);
			btnClientStop->Enable(! state);
			break;
		case MODE_SERVER_START:
			edtServerPort->Enable(state);
			chkServerPassive->Enable(state);
			btnServerStart->Enable(state);
			btnServerStop->Enable(! state);
			break;
	}
}

void FileRepairDlg::StatusMessage(wxCommandEvent& event)
{
	edtStatus->SetLabel(event.GetString());
}

void FileRepairDlg::ProgressMessage(wxCommandEvent& event)
{
	barProgress->SetValue(event.GetInt());
}


void FileRepairDlg::ServerMessage(wxCommandEvent& event)
{
	switch (tabMode->GetSelection()) {
		case 1:
			labClientResults->SetLabel(event.GetString());
			break;
		case 2:
			labServerResults->SetLabel(event.GetString());
			break;
	}
}

void FileRepairDlg::HashMessage(wxCommandEvent& event)
{
	edtHashResult->SetValue(event.GetString());
}

void FileRepairDlg::ErrorMessage(wxCommandEvent& event)
{
	wxMessageDialog	*errdlg = new wxMessageDialog(this, event.GetString(), wxT("Error"), wxOK | wxCENTRE | wxICON_HAND);
	errdlg->ShowModal();
	delete errdlg;
}

void FileRepairDlg::AlertMessage(wxCommandEvent& event)
{
	wxMessageDialog	*errdlg = new wxMessageDialog(this, event.GetString(), wxT(ALRT_ERR_TITLE), wxOK | wxCENTRE | wxICON_INFORMATION);
	errdlg->ShowModal();
	delete errdlg;
}

void FileRepairDlg::OnQuit(wxCommandEvent& WXUNUSED(event))
{
	if (frHelp->GetFrame())
		frHelp->GetFrame()->Close(TRUE);
	delete wxConfig::Set(NULL);
	Destroy();
}

void FileRepairDlg::OnClose(wxCloseEvent& WXUNUSED(event))
{
	OnQuit((wxCommandEvent &) wxEVT_NULL);
}

// Tha about box - provides program details and author contact info
void FileRepairDlg::About(wxCommandEvent& WXUNUSED(event))
{
	wxString msg;
	msg.Printf(wxT("%s version %s\n")
	           wxT("(C) %s by %s\n\n")
	           wxT("Distributed as %s\n")
	           wxT("Written using %s\n\n")
	           wxT("Please contact <%s> with any bugs, suggestions or comments\n"),
	           wxT(PROGRAM), wxT(VERSION), wxT(CONCEPTDATE), wxT(AUTHOR),
		   wxT(XWARE), wxVERSION_STRING, wxT(EMAIL));

	wxMessageBox(msg, wxT("About"), wxOK | wxICON_INFORMATION, this);
}

void FileRepairDlg::HelpContents(wxCommandEvent& WXUNUSED(event))
{
	frHelp->DisplayContents();
}

void FileRepairDlg::HelpIndex(wxCommandEvent& WXUNUSED(event))
{
	frHelp->DisplayIndex();
}
