/*
 * (c) Copyright Robin Hill 2002
 *
 * For license terms, see the file COPYING in this directory.
 */

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
	#pragma hdrstop
#endif

// For all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers
#ifndef WX_PRECOMP
	#include <wx/wx.h>
#endif

#include "wx_extra_functions.h"

// Find the absolute path where this application has been run from.
// argv0 is wxTheApp->argv[0]
// cwd is the current working directory (at startup)
// appVariableName is the name of a variable containing the directory for
//   this app, e.g. MYAPPDIR. This is checked first.
wxString wxFindAppPath(const wxString& argv0, const wxString& cwd, const wxString& appVariableName)
{
	wxString str;
// Try appVariableName
	if (!appVariableName.IsEmpty()) {
		str = wxGetenv(appVariableName);
		if (!str.IsEmpty())
			return str;
	}
	if (wxIsAbsolutePath(argv0))
		return wxPathOnly(argv0);
	else {
// Is it a relative path?
		wxString currentDir(cwd);
		if (currentDir.Last() != wxFILE_SEP_PATH)
			currentDir += wxFILE_SEP_PATH;
		str = currentDir + argv0;
		if (wxFileExists(str))
			return wxPathOnly(str);
	}
// OK, it's neither an absolute path nor a relative path.
// Search PATH.
	wxPathList pathList;
	pathList.AddEnvList(wxT("PATH"));
	str = pathList.FindAbsoluteValidPath(argv0);
	if (!str.IsEmpty())
		return wxPathOnly(str);
// Failed
	return wxEmptyString;
}
